==> maDB_1.4.0.tar.gz: maDB/R/SearchPattern.R <==
##
# search for a gene with a similar expression / regulation pattern like the submitted one...
dbSearchSimilarPattern <- function(con,template=NULL,id=NULL,compare.ids=NULL,include.by.pk=NULL,include.by.name=NULL,exclude.by.pk=NULL,exclude.by.name=NULL,what="E",chip="hgu133plus2",include.values=TRUE,v=TRUE,distance.fun=distance.euclidian,orientation="h",column.names=c("name","tissue","exposure_time"),order=TRUE){
	ids <- compare.ids
	if(what=="E"){
		info <- dbGetResult(dbSendQuery(con,"SELECT * FROM signal_channels JOIN samples ON (samples_fk=samples_pk)"))
		info.red <- .reduceInfo(info,include.by.name=include.by.name,include.by.pk=include.by.pk,exclude.by.name=exclude.by.name,exclude.by.pk=exclude.by.pk,pk.col="signal_channels_pk",order=order)
		if(chip=="hgu133plus2"){
			info.red <- info.red[info.red[,"species"]=="hs",]
		}
		if(chip=="mouse4302"){
			info.red <- info.red[info.red[,"species"]=="mm",]
		}
		if(is.null(ids)){
			ids <- dbGetResult(dbSendQuery(con,"SELECT id FROM annotation WHERE chip=",rpgsql.format.values(chip)))[,1]
		}
		if(is.null(template) & !is.null(id)){
			template <- getEDB(con,ids=id,signal.channels.pk=info.red[,"signal_channels_pk"],column.names=column.names,v=v)
		}
		HugeTable <- getEDB(con,ids=ids,signal.channels.pk=info.red[,"signal_channels_pk"],column.names=column.names,v=v)
		similarity <- calculateSimilarity(vector=template,matrix=HugeTable,include.values=include.values,distance.fun=distance.fun,orientation=orientation)
	}
	if(what=="M"){
		info <- dbGetResult(dbSendQuery(con,"SELECT * FROM comparisons JOIN signal_channels ON (r_signal_channel=signal_channels_pk) JOIN samples ON (samples_fk=samples_pk)"))
		info.red <- .reduceInfo(info,include.by.name=include.by.name,include.by.pk=include.by.pk,exclude.by.name=exclude.by.name,exclude.by.pk=exclude.by.pk,pk.col="comparisons_pk",order=order)
		if(chip=="hgu133plus2"){
			info.red <- info.red[info.red[,"species"]=="hs",]
		}
		if(chip=="mouse4302"){
			info.red <- info.red[info.red[,"species"]=="mm",]
		}
		if(is.null(ids)){
			ids <- dbGetResult(dbSendQuery(con,"SELECT id FROM annotation WHERE chip=",rpgsql.format.values(chip)))[,1]
		}
		if(is.null(template) & !is.null(id)){
			template <- getMDB(con,ids=id,comparisons.pk=info.red[,"comparisons_pk"],column.names=column.names,v=v)
			template <- template[2:length(template)]
		}
		HugeTable <- getMDB(con,ids=ids,comparisons.pk=info.red[,"comparisons_pk"],column.names=column.names,v=v)
		#write.table(HugeTable,file="MDB1.txt",sep="\t")
		
		similarity <- calculateSimilarity(vector=template,matrix=HugeTable[,2:length(HugeTable[1,])],include.values=include.values,distance.fun=distance.fun,orientation=orientation)

	}
	#create an object of the type Similarity
	Similarity <- new("Similarity")
	#Similarity@distance.metric=distance.fun
	sim <- as.numeric(similarity[,2])
	names(sim) <- similarity[,1]
	Similarity@distances <- sim
	if(include.values){
		Similarity@data <- similarity[,3:length(similarity[1,])]
		rownames(Similarity@data) <- names(sim)
	}
	if(class(template)=="matrix" | class(template)=="character"){
		templ <- as.numeric(template)
		names(templ) <- colnames(template)
		Similarity@template <- templ
	}
	else{
		Similarity@template <- template
	}
	if(!is.null(id)){
		Similarity@template.id <- id
	}
	Similarity
}






##
# calculates the similarity...
calculateSimilarity <- function(vector,matrix,orientation="h",distance.fun=distance.euclidian,include.values=FALSE){
	if(orientation=="v"){
		matrix <- t(matrix)
	}
	#write.table(matrix,file="matix1.txt",sep="\t")
	#write.table(vector,file="vector1.txt",sep="\t")
#	if(mode=="h"){
	if(length(vector)!=length(matrix[1,])){
		stop("The vector and the rows of the matrix have different dimensions!")
	}
	ncols <- 2
	if(include.values){
		ncols <- (2+length(matrix[1,]))
	}
	similarity <- matrix(ncol=ncols,nrow=length(matrix[,1]))
	similarity[,1] <- rownames(matrix)
	if(include.values){
		colnames(similarity) <- c("id","distance",c(colnames(matrix)))
	}
	else{
		colnames(similarity) <- c("id","distance")
	}
	#cat(dim(similarity),"\n")
	for(i in 1:length(similarity[,1])){
		similarity[i,2] <- distance.fun(as.numeric(vector),as.numeric(matrix[i,]))
		
	}
	if(include.values){
		similarity[,3:length(similarity[1,])] <- matrix
	}
#	}
#	if(mode=="fv"){
#		if(length(vector)!=length(matrix[,1])){
#			stop("The vector and the columns of the matrix have different dimensions!")
#		}
#		ncols <- 2
#		if(include.values){
#			ncols <- (2+length(matrix[,1]))
#		}
#		similarity <- matrix(ncol=ncols,nrow=length(matrix[1,]))
#		similarity[,1] <- colnames(matrix)
#		rownames(similarity) <- similarity[,1]
#		if(include.values){
#			colnames(similarity) <- c("id","distance",c(rownames(matrix)))
#		}
#		else{
#			colnames(similarity) <- c("id","distance")
#		}
#		for(i in 1:length(similarity[1,])){
#			similarity[i,2] <- distance.fun(vector,matrix[,i])
#		}
#	}
	idx <- order(as.numeric(similarity[,2]))
	r.names <- rownames(similarity)
	similarity <- similarity[idx,]
	rownames(similarity) <- r.names
	similarity
}

##
# euclidian distance between two vectors... 
distance.euclidian <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	d <- sqrt(sum((x-y)^2))
	d
}

distance.pearson <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	a <- x - mean(x)
	b <- y - mean(y)
	
	d <- sum(a*b) / (sqrt(sum(a^2)*sum(b^2)))
	d
}

distance.pearson.uncentered <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	a <- x - mean(x)
	b <- y - mean(y)
	
	d <- sum(x*y) / (sqrt(sum(a^2)*sum(b^2)))
	d
}

distance.spearman <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	Res <- cor.test(x,y,method="spearman")
	d <- Res$estimate
	d
}



plotSimilarity <- function(ids,distances,data,template,restrict.to=min(5,length(data[,1])),template.id="custom",legend=TRUE,...){
	data.sub <- data[1:restrict.to,]
	y.min <- min(as.numeric(data.sub))
	y.min <- min(as.numeric(template),y.min)
	y.max <- max(as.numeric(data.sub))
	y.max <- max(as.numeric(template),y.max)
	col.names <- colnames(data)
	x.max <- length(data.sub[1,])
	if(legend){
		x.max <- x.max+x.max/4
	}
	
	plot(template,col=1,lwd=2,xlim=c(0,x.max),ylim=c(y.min,(y.max+(y.max-y.min)/3)),type="l",...)
	text(x=seq(1:length(data[1,])),y=(y.max),label=col.names,srt=90,adj=0,...)
	for(i in 1:restrict.to){
		points(data.sub[i,],col=(i+1),type="l",...)
	}
	points(template,col=1,lwd=2,type="l",...)
	if(legend){
		legend(length(data.sub[1,]),(y.max),legend=paste(c(template.id,ids[1:restrict.to])),col=seq(1:(restrict.to+1)),pch=15,...)
	}
}


setClass("Similarity",representation(distances="numeric",distance.metric="character",data="matrix",template="numeric",template.id="character"),
	prototype=list(template.id="custom",distance.metric="none",data=NULL)
)


if( !isGeneric("plot") )
	setGeneric("plot", function(x,y,...)
	standardGeneric("plot"))

#################################################################################################
setMethod("plot","Similarity",
	function(x,y,...){
		if(is.null(x@data)){
			stop("can not plot any data! the @data slot is empty!\n\n")
		}
		plotSimilarity(ids=names(x@distances),distances=x@distances,data=x@data,template=x@template,template.id=x@template.id,...)
	})


if( !isGeneric("getData") )
	setGeneric("getData", function(object,...)
	standardGeneric("getData"))

#################################################################################################
setMethod("getData","Similarity",
	function(object,...){
		object@data
	})


if( !isGeneric("getDistances") )
	setGeneric("getDistances", function(object,...)
	standardGeneric("getDistances"))

#################################################################################################
setMethod("getDistances","Similarity",
	function(object,...){
		object@ddistances
	})


if( !isGeneric("print") )
	setGeneric("print", function(x,...)
	standardGeneric("print"))

#################################################################################################
setMethod("print","Similarity",
	function(x,...){
		cat("An object from the type Similarity\n\n")
		cat("@distance.metric\n")
		cat(x@distance.metric)
		cat("\n\n")
		cat("@template\n")
		cat(x@template,"\n\n")
		cat("@template.id\n")
		cat(x@template.id,"\n\n")
		cat("@distances\n")
		print(x@distances[1:5])
		cat("...\n\n")
		if(!is.null(dim(x@data))){
			cat("@data\n")
			until <- min(length(x@data[,1]),5)
			print(x@data[1:until,])
			cat("...\n")
		}
		cat("\n")
	})


setMethod("sort","Similarity",
	function(x,partial=NULL,na.last=NA,decreasing=FALSE,method=c("shell","quick"),index.return=FALSE){
		x.new <- x
		idx <- order(x@distances,na.last=na.last,decreasing=decreasing)
		if(!is.null(dim(x@data))){
			x.new@data <- x@data[idx,]
		}
		x.new@distances <- x@distances[idx]
		x.new
	})

==> maDB_1.6.0.tar.gz: maDB/R/SearchPattern.R <==
##
# search for a gene with a similar expression / regulation pattern like the submitted one...
# 26.04.2005: possible bug resolved: restrict search to selected chip!
dbSearchSimilarPattern <- function(con,template=NULL,id=NULL,compare.ids=NULL,include.by.pk=NULL,include.by.name=NULL,exclude.by.pk=NULL,exclude.by.name=NULL,what="E",chip="hgu133plus2",include.values=TRUE,v=TRUE,distance.fun=distance.euclidian,orientation="h",column.names=c("name","tissue","exposure_time"),order=TRUE){
	ids <- compare.ids
	require(pgUtils)
	if(what=="E"){
		#info <- dbGetResult(dbSendQuery(con,"SELECT * FROM signal_channels JOIN samples ON (samples_fk=samples_pk)"))
		info <- dbGetResult(dbSendQuery(con,createJOINQuery(join="signal_channels",with="arrays",select="signal_channels_pk,chip,samples_fk,samples.*"),"JOIN samples ON (samples_fk=samples_pk)"))
		info.red <- .reduceInfo(info,include.by.name=include.by.name,include.by.pk=include.by.pk,exclude.by.name=exclude.by.name,exclude.by.pk=exclude.by.pk,pk.col="signal_channels_pk",order=order)
		
		info.red <- info.red[info.red[,"chip"]==chip,]
		#if(chip=="hgu133plus2"){
		#	info.red <- info.red[info.red[,"species"]=="hs",]
		#}
		#if(chip=="mouse4302"){
		#	info.red <- info.red[info.red[,"species"]=="mm",]
		#}
		if(is.null(ids)){
			ids <- dbGetResult(dbSendQuery(con,"SELECT id FROM annotation WHERE chip=",rpgsql.format.values(chip)))[,1]
		}
		if(is.null(template) & !is.null(id)){
			template <- getEDB(con,ids=id,signal.channels.pk=info.red[,"signal_channels_pk"],column.names=column.names,v=v)
		}
		HugeTable <- getEDB(con,ids=ids,signal.channels.pk=info.red[,"signal_channels_pk"],column.names=column.names,v=v)
		similarity <- calculateSimilarity(vector=template,matrix=HugeTable,include.values=include.values,distance.fun=distance.fun,orientation=orientation)
	}
	if(what=="M"){
		#info <- dbGetResult(dbSendQuery(con,"SELECT * FROM comparisons JOIN signal_channels ON (r_signal_channel=signal_channels_pk) JOIN samples ON (samples_fk=samples_pk)"))
		info <- dbGetResult(dbSendQuery(con,createJOINQuery(join="comparisons",with="arrays",select="arrays.chip,arrays.arrays_pk,comparisons.*,signal_channels.*,samples.*"),"JOIN samples ON (samples_fk=samples_pk)"))
		info.red <- .reduceInfo(info,include.by.name=include.by.name,include.by.pk=include.by.pk,exclude.by.name=exclude.by.name,exclude.by.pk=exclude.by.pk,pk.col="comparisons_pk",order=order)
		
		info.red <- info.red[info.red[,"chip"]==chip,]
		#if(chip=="hgu133plus2"){
		#	info.red <- info.red[info.red[,"species"]=="hs",]
		#}
		#if(chip=="mouse4302"){
		#	info.red <- info.red[info.red[,"species"]=="mm",]
		#}
		if(is.null(ids)){
			ids <- dbGetResult(dbSendQuery(con,"SELECT id FROM annotation WHERE chip=",rpgsql.format.values(chip)))[,1]
		}
		if(is.null(template) & !is.null(id)){
			template <- getMDB(con,ids=id,comparisons.pk=info.red[,"comparisons_pk"],column.names=column.names,v=v)
			template <- template[2:length(template)]
		}
		HugeTable <- getMDB(con,ids=ids,comparisons.pk=info.red[,"comparisons_pk"],column.names=column.names,v=v)
		#write.table(HugeTable,file="MDB1.txt",sep="\t")
		
		similarity <- calculateSimilarity(vector=template,matrix=HugeTable[,2:length(HugeTable[1,])],include.values=include.values,distance.fun=distance.fun,orientation=orientation)

	}
	#create an object of the type Similarity
	Similarity <- new("Similarity")
	#Similarity@distance.metric=distance.fun
	sim <- as.numeric(similarity[,2])
	names(sim) <- similarity[,1]
	Similarity@distances <- sim
	if(include.values){
		Similarity@data <- similarity[,3:length(similarity[1,])]
		rownames(Similarity@data) <- names(sim)
	}
	if(class(template)=="matrix" | class(template)=="character"){
		templ <- as.numeric(template)
		names(templ) <- colnames(template)
		Similarity@template <- templ
	}
	else{
		Similarity@template <- template
	}
	if(!is.null(id)){
		Similarity@template.id <- id
	}
	Similarity
}






##
# calculates the similarity...
calculateSimilarity <- function(vector,matrix,orientation="h",distance.fun=distance.euclidian,include.values=FALSE){
	if(orientation=="v"){
		matrix <- t(matrix)
	}
	#write.table(matrix,file="matix1.txt",sep="\t")
	#write.table(vector,file="vector1.txt",sep="\t")
#	if(mode=="h"){
	if(length(vector)!=length(matrix[1,])){
		stop("The vector and the rows of the matrix have different dimensions!")
	}
	ncols <- 2
	if(include.values){
		ncols <- (2+length(matrix[1,]))
	}
	similarity <- matrix(ncol=ncols,nrow=length(matrix[,1]))
	similarity[,1] <- rownames(matrix)
	if(include.values){
		colnames(similarity) <- c("id","distance",c(colnames(matrix)))
	}
	else{
		colnames(similarity) <- c("id","distance")
	}
	#cat(dim(similarity),"\n")
	for(i in 1:length(similarity[,1])){
		similarity[i,2] <- distance.fun(as.numeric(vector),as.numeric(matrix[i,]))
		
	}
	if(include.values){
		similarity[,3:length(similarity[1,])] <- matrix
	}
#	}
#	if(mode=="fv"){
#		if(length(vector)!=length(matrix[,1])){
#			stop("The vector and the columns of the matrix have different dimensions!")
#		}
#		ncols <- 2
#		if(include.values){
#			ncols <- (2+length(matrix[,1]))
#		}
#		similarity <- matrix(ncol=ncols,nrow=length(matrix[1,]))
#		similarity[,1] <- colnames(matrix)
#		rownames(similarity) <- similarity[,1]
#		if(include.values){
#			colnames(similarity) <- c("id","distance",c(rownames(matrix)))
#		}
#		else{
#			colnames(similarity) <- c("id","distance")
#		}
#		for(i in 1:length(similarity[1,])){
#			similarity[i,2] <- distance.fun(vector,matrix[,i])
#		}
#	}
	idx <- order(as.numeric(similarity[,2]))
	r.names <- rownames(similarity)
	similarity <- similarity[idx,]
	rownames(similarity) <- r.names
	similarity
}

##
# euclidian distance between two vectors... 
distance.euclidian <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	d <- sqrt(sum((x-y)^2))
	d
}

distance.pearson <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	a <- x - mean(x)
	b <- y - mean(y)
	
	d <- sum(a*b) / (sqrt(sum(a^2)*sum(b^2)))
	d
}

distance.pearson.uncentered <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	a <- x - mean(x)
	b <- y - mean(y)
	
	d <- sum(x*y) / (sqrt(sum(a^2)*sum(b^2)))
	d
}

distance.spearman <- function(x,y,na.rm=FALSE,...){
	if(na.rm){
		x <- x[!is.na(x) & !is.na(y)]
		y <- y[!is.na(x) & !is.na(y)]
	}
	Res <- cor.test(x,y,method="spearman")
	d <- Res$estimate
	d
}



plotSimilarity <- function(ids,distances,data,template,restrict.to=min(5,length(data[,1])),template.id="custom",legend=TRUE,...){
	data.sub <- data[1:restrict.to,]
	y.min <- min(as.numeric(data.sub))
	y.min <- min(as.numeric(template),y.min)
	y.max <- max(as.numeric(data.sub))
	y.max <- max(as.numeric(template),y.max)
	col.names <- colnames(data)
	x.max <- length(data.sub[1,])
	if(legend){
		x.max <- x.max+x.max/4
	}
	
	plot(template,col=1,lwd=2,xlim=c(0,x.max),ylim=c(y.min,(y.max+(y.max-y.min)/3)),type="l",...)
	text(x=seq(1:length(data[1,])),y=(y.max),label=col.names,srt=90,adj=0,...)
	for(i in 1:restrict.to){
		points(data.sub[i,],col=(i+1),type="l",...)
	}
	points(template,col=1,lwd=2,type="l",...)
	if(legend){
		legend(length(data.sub[1,]),(y.max),legend=paste(c(template.id,ids[1:restrict.to])),col=seq(1:(restrict.to+1)),pch=15,...)
	}
}


setClass("Similarity",representation(distances="numeric",distance.metric="character",data="matrix",template="numeric",template.id="character"),
	prototype=list(template.id="custom",distance.metric="none",data=NULL)
)


if( !isGeneric("plot") )
	setGeneric("plot", function(x,y,...)
	standardGeneric("plot"))

#################################################################################################
setMethod("plot","Similarity",
	function(x,y,...){
		if(is.null(x@data)){
			stop("can not plot any data! the @data slot is empty!\n\n")
		}
		plotSimilarity(ids=names(x@distances),distances=x@distances,data=x@data,template=x@template,template.id=x@template.id,...)
	})


if( !isGeneric("getData") )
	setGeneric("getData", function(object,...)
	standardGeneric("getData"))

#################################################################################################
setMethod("getData","Similarity",
	function(object,...){
		object@data
	})


if( !isGeneric("getDistances") )
	setGeneric("getDistances", function(object,...)
	standardGeneric("getDistances"))

#################################################################################################
setMethod("getDistances","Similarity",
	function(object,...){
		object@ddistances
	})


if( !isGeneric("print") )
	setGeneric("print", function(x,...)
	standardGeneric("print"))

#################################################################################################
setMethod("print","Similarity",
	function(x,...){
		cat("An object from the type Similarity\n\n")
		cat("@distance.metric\n")
		cat(x@distance.metric)
		cat("\n\n")
		cat("@template\n")
		cat(x@template,"\n\n")
		cat("@template.id\n")
		cat(x@template.id,"\n\n")
		cat("@distances\n")
		print(x@distances[1:5])
		cat("...\n\n")
		if(!is.null(dim(x@data))){
			cat("@data\n")
			until <- min(length(x@data[,1]),5)
			print(x@data[1:until,])
			cat("...\n")
		}
		cat("\n")
	})


setMethod("sort","Similarity",
	function(x,partial=NULL,na.last=NA,decreasing=FALSE,method=c("shell","quick"),index.return=FALSE){
		x.new <- x
		idx <- order(x@distances,na.last=na.last,decreasing=decreasing)
		if(!is.null(dim(x@data))){
			x.new@data <- x@data[idx,]
		}
		x.new@distances <- x@distances[idx]
		x.new
	})
